@@ -6,7 +6,7 @@
 ;; Maintainer: Denote Development <~protesilaos/denote@lists.sr.ht>
 ;; URL: https://git.sr.ht/~protesilaos/denote
 ;; Mailing-List: https://lists.sr.ht/~protesilaos/denote
-;; Version: 0.2.1
+;; Version: 0.4.0
 ;; Package-Requires: ((emacs "27.2"))
 
 ;; This file is NOT part of GNU Emacs.
@@ -96,7 +96,7 @@
 ;;; Code:
 
 (require 'seq)
-(eval-when-compile (require 'cl-lib))
+(eval-when-compile (require 'subr-x))
 
 (defgroup denote ()
   "Simple notes with an efficient file-naming scheme."
@@ -128,6 +128,7 @@ the function `denote-directory' instead: it returns the path as a
 directory and also checks if a safe local value should be used."
   :group 'denote
   :safe (lambda (val) (or (eq val 'local) (eq val 'default-directory)))
+  :link '(info-link "(denote) Maintain separate directories for notes")
   :type 'directory)
 
 (defcustom denote-known-keywords
@@ -162,6 +163,80 @@ the appropriate list of strings."
   :group 'denote
   :type 'boolean)
 
+(defconst denote--prompt-symbols
+  '(title keywords date file-type subdirectory)
+  "List of symbols representing `denote' prompts.")
+
+(defcustom denote-prompts '(title keywords)
+  "Specify the prompts of the `denote' command for interactive use.
+
+The value is a list of symbols, which includes any of the following:
+
+- `title': Prompt for the title of the new note.
+
+- `keywords': Prompts with completion for the keywords of the new
+  note.  Available candidates are those specified in the user
+  option `denote-known-keywords'.  If the user option
+  `denote-infer-keywords' is non-nil, keywords in existing note
+  file names are included in the list of candidates.  The
+  `keywords' prompt uses `completing-read-multiple', meaning that
+  it can accept multiple keywords separated by a comma (or
+  whatever the value of `crm-separator' is).
+
+- `file-type': Prompts with completion for the file type of the
+  new note.  Available candidates are those specified in the user
+  option `denote-file-type'.  Without this prompt, `denote' uses
+  the value of `denote-file-type'.
+
+- `subdirectory': Prompts with completion for a subdirectory in
+  which to create the note.  Available candidates are the value
+  of the user option `denote-directory' and all of its
+  subdirectories.  Any subdirectory must already exist: Denote
+  will not create it.
+
+- `date': Prompts for the date of the new note.  It will expect
+  an input like 2022-06-16 or a date plus time: 2022-06-16 14:30.
+  Without the `date' prompt, the `denote' command uses the
+  `current-time'.
+
+The prompts occur in the given order.
+
+If the value of this user option is nil, no prompts are used.
+The resulting file name will consist of an identifier (i.e. the
+date and time) and a supported file type extension (per
+`denote-file-type').
+
+Recall that Denote's standard file-naming scheme is defined as
+follows (read the manual for the technicalities):
+
+    DATE--TITLE__KEYWORDS.EXT
+
+If either or both of the `title' and `keywords' prompts are not
+included in the value of this variable, file names will be any of
+those permutations:
+
+    DATE.EXT
+    DATE--TITLE.EXT
+    DATE__KEYWORDS.EXT
+
+When in doubt, always include the `title' and `keywords' prompts.
+
+Finally, this user option only affects the interactive use of the
+`denote' command (advanced users can call it from Lisp).  For
+ad-hoc interactive actions that do not change the default
+behaviour of the `denote' command, users can invoke these
+convenience commands: `denote-type', `denote-subdirectory',
+`denote-date'."
+  :group 'denote
+  :link '(info-link "(denote) The denote-prompts option")
+  :type '(radio (const :tag "Use no prompts" nil)
+                (set :tag "Available prompts" :greedy t
+                     (const :tag "Title" title)
+                     (const :tag "Keywords" keywords)
+                     (const :tag "Date" date)
+                     (const :tag "File type extension" file-type)
+                     (const :tag "Subdirectory" subdirectory))))
+
 (defcustom denote-sort-keywords t
   "Whether to sort keywords in new files.
 
@@ -233,28 +308,33 @@ are described in the doc string of `format-time-string'."
 
 ;;;; Main variables
 
+;; For character classes, evaluate: (info "(elisp) Char Classes")
 (defconst denote--id-format "%Y%m%dT%H%M%S"
   "Format of ID prefix of a note's filename.")
 
 (defconst denote--id-regexp "\\([0-9]\\{8\\}\\)\\(T[0-9]\\{6\\}\\)"
   "Regular expression to match `denote--id-format'.")
 
-(defconst denote--file-title-regexp
-  (concat denote--id-regexp "\\(--\\)\\(.*\\)\\(__\\)")
-  "Regular expression to match file names from `denote'.")
+(defconst denote--title-regexp "--\\([[:alnum:][:nonascii:]-]*\\)"
+  "Regular expression to match the title field.")
 
-(defconst denote--file-regexp
-  (concat denote--file-title-regexp "\\([0-9A-Za-z_-]*\\)\\(\\.?.*\\)")
-  "Regular expression to match the entire file name'.")
+(defconst denote--keywords-regexp "__\\([[:alnum:][:nonascii:]_-]*\\)"
+  "Regular expression to match keywords.")
 
-(defconst denote--file-only-note-regexp
-  (concat denote--file-regexp "\\.\\(org\\|md\\|txt\\)")
-  "Regular expression to match the entire file name of a note file.")
+(defconst denote--extension-regexp "\\.\\(org\\|md\\|txt\\)"
+  "Regular expression to match supported Denote extensions.")
 
 (defconst denote--punctuation-regexp "[][{}!@#$%^&*()_=+'\"?,.\|;:~`‘’“”/]*"
-  "Regular expression of punctionation that should be removed.
+  "Punctionation that is removed from file names.
 We consider those characters illigal for our purposes.")
 
+(defvar denote-punctuation-excluded-extra-regexp nil
+  "Additional punctuation that is removed from file names.
+This variable is for advanced users who need to extend the
+`denote--punctuation-regexp'.  Once we have a better
+understanding of what we should be omitting, we will update
+things accordingly.")
+
 (defvar denote-last-path nil "Store last path.")
 (defvar denote-last-title nil "Store last title.")
 (defvar denote-last-keywords nil "Store last keywords.")
@@ -277,21 +357,13 @@ We consider those characters illigal for our purposes.")
          (path (if (or (eq val 'default-directory) (eq val 'local)) default-directory val)))
     (unless (file-directory-p path)
       (make-directory path t))
-    (file-name-as-directory path)))
-
-(defun denote--extract (regexp str &optional group)
-  "Extract REGEXP from STR, with optional regexp GROUP."
-  (when group
-    (unless (and (integerp group) (> group 0))
-      (error "`%s' is not a positive integer" group)))
-  (with-temp-buffer
-    (insert str)
-    (when (re-search-forward regexp nil t -1)
-      (match-string (or group 1)))))
+    (file-name-as-directory (expand-file-name path))))
 
 (defun denote--slug-no-punct (str)
   "Convert STR to a file name slug."
-  (replace-regexp-in-string denote--punctuation-regexp "" str))
+  (replace-regexp-in-string
+   (concat denote--punctuation-regexp denote-punctuation-excluded-extra-regexp)
+   "" str))
 
 (defun denote--slug-hyphenate (str)
   "Replace spaces with hyphens in STR.
@@ -316,15 +388,10 @@ trailing hyphen."
 
 (defun denote--sluggify-keywords (keywords)
   "Sluggify KEYWORDS."
-  (cond
-   ((listp keywords)
-    (if denote-allow-multi-word-keywords
-        (mapcar #'denote--sluggify keywords)
-      (mapcar #'denote--sluggify-and-join keywords)))
-   (t
-    (if denote-allow-multi-word-keywords
-        (denote--sluggify keywords)
-      (denote--sluggify-and-join keywords)))))
+  (mapcar (if denote-allow-multi-word-keywords
+              #'denote--sluggify
+            #'denote--sluggify-and-join)
+          keywords))
 
 (defun denote--file-empty-p (file)
   "Return non-nil if FILE is empty."
@@ -333,19 +400,22 @@ trailing hyphen."
 (defun denote--only-note-p (file)
   "Make sure FILE is an actual Denote note.
 FILE is relative to the variable `denote-directory'."
-  (and (not (file-directory-p file))
-       (file-regular-p file)
-       (string-match-p denote--file-only-note-regexp file)
-       (not (string-match-p "[#~]\\'" file))))
+  (let ((file-name (file-name-nondirectory file)))
+    (and (not (file-directory-p file))
+         (file-regular-p file)
+         (string-match-p (concat "\\`" denote--id-regexp
+                                 ".*" denote--extension-regexp "\\'")
+                         file-name)
+         (not (string-match-p "[#~]\\'" file)))))
 
 (defun denote--file-name-relative-to-denote-directory (file)
   "Return file name of FILE relative to the variable `denote-directory'.
 FILE must be an absolute path."
-  (if-let* ((dir (denote-directory))
-            ((file-name-absolute-p file))
-            ((string-prefix-p dir file)))
-      (substring-no-properties file (length dir))
-    file))
+  (when-let* ((dir (denote-directory))
+              ((file-name-absolute-p file))
+              (file-name (expand-file-name file))
+              ((string-prefix-p dir file-name)))
+    (substring-no-properties file-name (length dir))))
 
 (defun denote--current-file-is-note-p ()
   "Return non-nil if current file likely is a Denote note."
@@ -376,48 +446,39 @@ names that are relative to the variable `denote-directory'."
        (lambda (s) (denote--file-name-relative-to-denote-directory s))
        files))))
 
-(declare-function cl-find-if "cl-seq" (cl-pred cl-list &rest cl-keys))
-
 (defun denote--get-note-path-by-id (id)
   "Return the absolute path of ID note in variable `denote-directory'."
-  (cl-find-if
+  (seq-find
    (lambda (f)
      (string-prefix-p id (file-name-nondirectory f)))
    (denote--directory-files :absolute)))
 
-(defun denote--directory-files-matching-regexp (regexp &optional no-check-current)
-  "Return list of files matching REGEXP.
-With optional NO-CHECK-CURRENT do not test if the current file is
-part of the list."
+(defun denote--directory-files-matching-regexp (regexp)
+  "Return list of files matching REGEXP."
   (delq
    nil
    (mapcar
     (lambda (f)
       (when (and (denote--only-note-p f)
                  (string-match-p regexp f)
-                 (or no-check-current
-                     (not (string= (file-name-nondirectory (buffer-file-name)) f))))
+                 (not (string= (file-name-nondirectory (buffer-file-name)) f)))
         f))
     (denote--directory-files))))
 
-(defun denote--keywords-in-files ()
-  "Produce list of keywords in `denote--directory-files'."
-  (delq nil (mapcar
-             (lambda (x)
-               (denote--extract denote--file-regexp x 6))
-             ;; REVIEW 2022-07-03: I tested this with ~3000 files.  It
-             ;; has about 2 seconds of delay on my end.  After I placed
-             ;; the list of those files in a variable instead of calling
-             ;; `denote--directory-files', there was no noticeable
-             ;; performance penalty.
-             (denote--directory-files))))
+(defun denote--extract-keywords-from-path (path)
+  "Extract keywords from PATH."
+  (let* ((file-name (file-name-nondirectory path))
+         (kws (when (string-match denote--keywords-regexp file-name)
+                (match-string-no-properties 1 file-name))))
+    (when kws
+      (split-string kws "_"))))
 
 (defun denote--inferred-keywords ()
   "Extract keywords from `denote--directory-files'."
-  (let ((sequence (denote--keywords-in-files)))
-    (mapcan (lambda (s)
-              (split-string s "_" t))
-            sequence)))
+  (delete-dups
+   (mapcan (lambda (p)
+             (denote--extract-keywords-from-path p))
+           (denote--directory-files))))
 
 (defun denote-keywords ()
   "Return appropriate list of keyword candidates.
@@ -434,9 +495,10 @@ existing notes and combine them into a list with
 
 (defun denote--keywords-crm (keywords)
   "Use `completing-read-multiple' for KEYWORDS."
-  (completing-read-multiple
-   "File keyword: " keywords
-   nil nil nil 'denote--keyword-history))
+  (delete-dups
+   (completing-read-multiple
+    "File keyword: " keywords
+    nil nil nil 'denote--keyword-history)))
 
 (defun denote--keywords-prompt ()
   "Prompt for one or more keywords.
@@ -445,34 +507,19 @@ In the case of multiple entries, those are separated by the
 output is sorted with `string-lessp'."
   (let ((choice (denote--keywords-crm (denote-keywords))))
     (setq denote-last-keywords
-          (cond
-           ((null choice)
-            "")
-           ((= (length choice) 1)
-            (car choice))
-           ((if denote-sort-keywords
-                (sort choice #'string-lessp)
-              choice))))))
+          (if denote-sort-keywords
+              (sort choice #'string-lessp)
+            choice))))
 
 (defun denote--keywords-combine (keywords)
   "Format KEYWORDS output of `denote--keywords-prompt'."
-  (if (and (> (length keywords) 1)
-           (not (stringp keywords)))
-      (mapconcat #'downcase keywords "_")
-    keywords))
+  (mapconcat #'downcase keywords "_"))
 
 (defun denote--keywords-add-to-history (keywords)
   "Append KEYWORDS to `denote--keyword-history'."
-  (if-let ((listed (listp keywords))
-           (length (length keywords)))
-      (cond
-       ((and listed (= length 1))
-        (car keywords))
-       ((and listed (> length 1))
-        (mapc (lambda (kw)
-                (add-to-history 'denote--keyword-history kw))
-              (delete-dups keywords))))
-    (add-to-history 'denote--keyword-history keywords)))
+  (mapc (lambda (kw)
+          (add-to-history 'denote--keyword-history kw))
+        (delete-dups keywords)))
 
 ;;;; New note
 
@@ -486,16 +533,25 @@ output is sorted with `string-lessp'."
     ('text ".txt")
     (_ ".org")))
 
-(defun denote--format-file (path id keywords slug extension)
+(defun denote--format-file (path id keywords title-slug extension)
   "Format file name.
-PATH, ID, KEYWORDS, SLUG are expected to be supplied by `denote'
-or equivalent: they will all be converted into a single string.
-EXTENSION is the file type extension, either a string which
-include the starting dot or the return value of
+PATH, ID, KEYWORDS, TITLE-SLUG are expected to be supplied by
+`denote' or equivalent: they will all be converted into a single
+string.  EXTENSION is the file type extension, either a string
+which include the starting dot or the return value of
 `denote--file-extension'."
   (let ((kws (denote--keywords-combine keywords))
-        (ext (or extension (denote--file-extension))))
-    (format "%s%s--%s__%s%s" path id slug kws ext)))
+        (ext (or extension (denote--file-extension)))
+        (empty-title (string-empty-p title-slug)))
+    (cond
+     ((and keywords title-slug (not empty-title))
+      (format "%s%s--%s__%s%s" path id title-slug kws ext))
+     ((and keywords empty-title)
+      (format "%s%s__%s%s" path id kws ext))
+     ((and title-slug (not empty-title))
+      (format "%s%s--%s%s" path id title-slug ext))
+     (t
+      (format "%s%s%s" path id ext)))))
 
 (defun denote--map-quote-downcase (seq)
   "Quote and downcase elements in SEQ."
@@ -513,15 +569,9 @@ With optional TYPE, format the keywords accordingly (this might
 be `toml' or, in the future, some other spec that needss special
 treatment)."
   (let ((kw (denote--sluggify-keywords keywords)))
-    (cond
-     ((and (> (length kw) 1) (not (stringp kw)))
-      (pcase type
-        ('toml (format "[%s]" (denote--map-quote-downcase kw)))
-        (_ (mapconcat #'downcase kw "  "))))
-     (t
-      (pcase type
-        ('toml (format "[%S]" (downcase kw)))
-        (_ (downcase kw)))))))
+    (pcase type
+      ('markdown-toml (format "[%s]" (denote--map-quote-downcase kw)))
+      (_ (mapconcat #'downcase kw "  ")))))
 
 (defvar denote-toml-front-matter
   "+++
@@ -589,7 +639,7 @@ TITLE, DATE, KEYWORDS, FILENAME, ID are all strings which are
 Optional FILETYPE is one of the values of `denote-file-type',
 else that variable is used."
   (let ((kw-space (denote--file-meta-keywords keywords))
-        (kw-toml (denote--file-meta-keywords keywords 'toml)))
+        (kw-toml (denote--file-meta-keywords keywords 'markdown-toml)))
     (pcase (or filetype denote-file-type)
       ('markdown-toml (format denote-toml-front-matter title date kw-toml id))
       ('markdown-yaml (format denote-yaml-front-matter title date kw-space id))
@@ -642,23 +692,21 @@ With optional DATE, use it else use the current one."
      (t
       (denote--date-org-timestamp date)))))
 
-(defun denote--prepare-note (title keywords &optional path date id)
-  "Use TITLE and KEYWORDS to prepare new note file.
-Use optional PATH, else create it with `denote--path'.  When PATH
-is provided, refrain from writing to a buffer (useful for org
-capture).
+(defun denote--prepare-note (title keywords date id directory file-type)
+  "Prepare a new note file.
 
-Optional DATE is passed to `denote--date', while optional ID is
-used to construct the path's identifier."
-  (let* ((default-directory (denote-directory))
-         (p (or path (denote--path title keywords default-directory id)))
-         (buffer (unless path (find-file p)))
+Arguments TITLE, KEYWORDS, DATE, ID, DIRECTORY, and FILE-TYPE
+should be valid for note creation."
+  (let* ((default-directory directory)
+         (denote-file-type file-type)
+         (path (denote--path title keywords default-directory id))
+         (buffer (find-file path))
          (header (denote--file-meta-header
                   title (denote--date date) keywords
-                  (format-time-string denote--id-format date))))
-    (unless path
-      (with-current-buffer buffer (insert header))
-      (setq denote-last-buffer buffer))
+                  (format-time-string denote--id-format date)
+                  file-type)))
+    (with-current-buffer buffer (insert header))
+    (setq denote-last-buffer buffer)
     (setq denote-last-front-matter header)))
 
 (defvar denote--title-history nil
@@ -676,27 +724,59 @@ Optional DEFAULT-TITLE is used as the default value."
 
 ;;;;; The `denote' command
 
+(defun denote--dir-in-denote-directory-p (directory)
+  "Return DIRECTORY if in variable `denote-directory', else nil."
+  (when-let* ((dir directory)
+              ((string-prefix-p (expand-file-name (denote-directory))
+                                (expand-file-name dir))))
+    dir))
+
 ;;;###autoload
-(defun denote (title keywords)
-  "Create new note with the appropriate metadata and file name.
+(defun denote (&optional title keywords file-type subdirectory date)
+  "Create a new note with the appropriate metadata and file name.
 
-This command first prompts for a file TITLE and then for one or
-more KEYWORDS (separated by the `crm-separator', typically a
-comma).  The latter supports completion though any arbitrary
-string can be inserted.
+When called interactively, the metadata and file name are prompted
+according to the value of `denote-prompts'.
 
-Completion candidates are those of `denote-known-keywords'.  If
-`denote-infer-keywords' is non-nil, then keywords in existing
-file names are also provided as candidates.
+When called from Lisp, all arguments are optional.
 
-When `denote-sort-keywords' is non-nil, keywords are sorted
-alphabetically in both the file name and file contents."
+- TITLE is a string or a function returning a string.
+
+- KEYWORDS is a list of strings.  The list can be empty or the
+  value can be set to nil.
+
+- FILE-TYPE is a symbol among those described in `denote-file-type'.
+
+- SUBDIRECTORY is a string representing the path to either the
+  value of the variable `denote-directory' or a subdirectory
+  thereof.  The subdirectory must exist: Denote will not create
+  it.  If SUBDIRECTORY does not resolve to a valid path, the
+  variable `denote-directory' is used instead.
+
+- DATE is a string representing a date like 2022-06-30 or a date
+  and time like 2022-06-16 14:30.  A nil value or an empty string
+  is interpreted as the `current-time'."
   (interactive
-   (list
-    (denote--title-prompt)
-    (denote--keywords-prompt)))
-  (denote--prepare-note title keywords)
-  (denote--keywords-add-to-history keywords))
+   (let ((args (make-vector 5 nil)))
+     (dolist (prompt denote-prompts)
+       (pcase prompt
+         ('title (aset args 0 (denote--title-prompt)))
+         ('keywords (aset args 1 (denote--keywords-prompt)))
+         ('file-type (aset args 2 (denote--file-type-prompt)))
+         ('subdirectory (aset args 3 (denote--subdirs-prompt)))
+         ('date (aset args 4 (denote--date-prompt)))))
+     (append args nil)))
+  (let* ((file-type (denote--file-type-symbol (or file-type denote-file-type)))
+         (date (if (or (null date) (string-empty-p date))
+                   (current-time)
+                 (denote--valid-date date)))
+         (id (format-time-string denote--id-format date))
+         (directory (if (denote--dir-in-denote-directory-p subdirectory)
+                        (file-name-as-directory subdirectory)
+                      (denote-directory))))
+    (denote--barf-duplicate-id id)
+    (denote--prepare-note (or title "") keywords date id directory file-type)
+    (denote--keywords-add-to-history keywords)))
 
 (defalias 'denote-create-note (symbol-function 'denote))
 
@@ -724,14 +804,14 @@ here for clarity."
    (t (user-error "`%s' is not a symbol or string" filetype))))
 
 ;;;###autoload
-(defun denote-type (filetype)
-  "Like `denote' but with FILETYPE for `denote-file-type'.
-In practice, this command lets you create, say, a Markdown file
-even when your default is Org.
+(defun denote-type ()
+  "Create note while prompting for a file type.
 
-When called from Lisp the FILETYPE must be a symbol."
-  (interactive (list (denote--file-type-prompt)))
-  (let ((denote-file-type (denote--file-type-symbol filetype)))
+This is the equivalent to calling `denote' when `denote-prompts'
+is set to \\='(file-type title keywords)."
+  (declare (interactive-only t))
+  (interactive)
+  (let ((denote-prompts '(file-type title keywords)))
     (call-interactively #'denote)))
 
 (defalias 'denote-create-note-using-type (symbol-function 'denote-type))
@@ -747,50 +827,72 @@ When called from Lisp the FILETYPE must be a symbol."
    "DATE and TIME for note (e.g. 2022-06-16 14:30): "
    nil 'denote--date-history))
 
+(defun denote--date-add-current-time (date)
+  "Add current time to DATE, if necessary.
+The idea is to turn 2020-01-15 into 2020-01-15 16:19 so that the
+hour and minute component is not left to 00:00.
+
+This reduces the burden on the user who would otherwise need to
+input that value in order to avoid the error of duplicate
+identifiers.
+
+It also addresses a difference between Emacs 28 and Emacs 29
+where the former does not read dates without a time component."
+  (if (<= (length date) 10)
+      (format "%s %s" date (format-time-string "%H:%M:%S" (current-time)))
+    date))
+
 (defun denote--valid-date (date)
   "Return DATE if parsed by `date-to-time', else signal error."
-  (date-to-time date))
+  (let ((datetime (denote--date-add-current-time date)))
+    (date-to-time datetime)))
+
+(defun denote--buffer-file-names ()
+  "Return file names of active buffers."
+  (mapcar
+   (lambda (name)
+     (file-name-nondirectory name))
+   (seq-filter
+    (lambda (name) (denote--only-note-p name))
+    (delq nil
+          (mapcar
+           (lambda (buf)
+             (buffer-file-name buf))
+           (buffer-list))))))
 
 ;; This should only be relevant for `denote-date', otherwise the
 ;; identifier is always unique (we trust that no-one writes multiple
 ;; notes within fractions of a second).
-(defun denote--id-exists-p (identifier no-check-current)
-  "Return non-nil if IDENTIFIER already exists.
-NO-CHECK-CURRENT passes the appropriate flag to
-`denote--directory-files-matching-regexp'."
-  (denote--directory-files-matching-regexp identifier no-check-current))
+(defun denote--id-exists-p (identifier)
+  "Return non-nil if IDENTIFIER already exists."
+  (let ((current-buffer-name (when (buffer-file-name)
+                               (file-name-nondirectory (buffer-file-name)))))
+    (or (seq-some (lambda (file)
+                    (string-match-p (concat "\\`" identifier) file))
+                  (delete current-buffer-name (denote--buffer-file-names)))
+        (delete current-buffer-name
+                (denote--directory-files-matching-regexp
+                 (concat "\\`" identifier))))))
 
 (defun denote--barf-duplicate-id (identifier)
   "Throw a user-error if IDENTIFIER already exists else return t."
-  (if (denote--id-exists-p identifier :no-check-current)
+  (if (denote--id-exists-p identifier)
       (user-error "`%s' already exists; aborting new note creation" identifier)
     t))
 
 ;;;###autoload
-(defun denote-date (date title keywords)
-  "Like `denote', but create new note for given DATE.
+(defun denote-date ()
+  "Create note while prompting for a date.
 
-DATE can either be something like 2022-06-16 or that plus time:
-2022-06-16 14:30.
+The date can be in YEAR-MONTH-DAY notation like 2022-06-30 or
+that plus the time: 2022-06-16 14:30
 
-The hour can be omitted, in which case it is interpreted as
-00:00.  Beware that you might create files with non-unique
-identifiers if they both have the same date and time.  In such a
-case, Denote will refrain from creating the new note.  Try with
-another DATE value where, for instance, a different time is
-specified.
-
-The TITLE and KEYWORDS arguments are the same as with `denote'."
-  (interactive
-   (list
-    (denote--date-prompt)
-    (denote--title-prompt)
-    (denote--keywords-prompt)))
-  (when-let ((d (denote--valid-date date))
-             (id (format-time-string denote--id-format d))
-             ((denote--barf-duplicate-id id)))
-    (denote--prepare-note title keywords nil d id)
-    (denote--keywords-add-to-history keywords)))
+This is the equivalent to calling `denote' when `denote-prompts'
+is set to \\='(date title keywords)."
+  (declare (interactive-only t))
+  (interactive)
+  (let ((denote-prompts '(date title keywords)))
+    (call-interactively #'denote)))
 
 (defalias 'denote-create-note-using-date (symbol-function 'denote-date))
 
@@ -829,22 +931,18 @@ The TITLE and KEYWORDS arguments are the same as with `denote'."
     (denote--subdirs-completion-table dirs)))
 
 ;;;###autoload
-(defun denote-subdirectory (directory title keywords)
-  "Like `denote' but ask for DIRECTORY to put the note in.
+(defun denote-subdirectory ()
+  "Create note while prompting for a subdirectory.
 
-The DIRECTORY is either the variable `denote-directory' or a
-subdirectory of it.  The TITLE and KEYWORDS are the same as for
-the `denote' command.
+Available candidates include the value of the variable
+`denote-directory' and any subdirectory thereof.
 
-Denote does not create subdirectories."
-  (interactive
-   (list
-    (denote--subdirs-prompt)
-    (denote--title-prompt)
-    (denote--keywords-prompt)))
-  (let ((denote-directory directory))
-    (denote--prepare-note title keywords)
-    (denote--keywords-add-to-history keywords)))
+This is equivalent to calling `denote' when `denote-prompts' is set to
+\\='(subdirectory title keywords)."
+  (declare (interactive-only t))
+  (interactive)
+  (let ((denote-prompts '(subdirectory title keywords)))
+    (call-interactively #'denote)))
 
 (defalias 'denote-create-note-in-subdirectory (symbol-function 'denote-subdirectory))
 
